package gs.slug.v0

import scala.util.matching.Regex

/** Restricted string intended to be used as a unique, URL-safe identifier
  * within some specific context (not a globally unique identifier). This type
  * adheres to the regular expression: `^[a-z0-9]+(?:\-[a-z0-9]+)*$`
  *
  * Essentially, Slugs follow these rules:
  *
  *   - Segments are defined as lowercase alphanumeric ASCII strings.
  *   - Segments are separated by exactly one `-` character.
  *   - At least one segment is required.
  *
  * If you need full unicode support, more characters, fewer constraints, etc.
  * don't use this type.
  */
opaque type Slug = String

object Slug:

  val SlugPattern: Regex = "^[a-z0-9]+(?:\\-[a-z0-9]+)*$".r

  /** Instantiate a [[Slug]] by validating the candidate input.
    *
    * @param candidate
    *   The candidate string used to create the [[Slug]].
    * @return
    *   The [[Slug]], or `None` if the candidate is invalid.
    */
  def validate(candidate: String): Option[Slug] =
    if SlugPattern.matches(candidate) then Some(candidate) else None

  given CanEqual[Slug, Slug] = CanEqual.derived

  extension (slug: Slug)
    /** Render this [[Slug]] as a string.
      *
      * @return
      *   The string representation of this [[Slug]].
      */
    def str(): String = slug
